@@ -52,3 +52,17 @@ def apicall(url, signkey, **params):
     body = urllib.urlencode(data)
     urllib2.urlopen(url, body)
 
+# taken from http://djangosnippets.org/snippets/564/
+from hashlib import sha1
+from django.core.cache import cache as _djcache
+def cache(seconds = 900):
+    def doCache(f):
+        def x(*args, **kwargs):
+                key = sha1(str(f.__module__) + str(f.__name__) + str(args) + str(kwargs)).hexdigest()
+                result = _djcache.get(key)
+                if result is None:
+                    result = f(*args, **kwargs)
+                    _djcache.set(key, result, seconds)
+                return result
+        return x
+    return doCache
\ No newline at end of file
