@@ -1,19 +1,17 @@
 import json
 import datetime
 import calendar
-
+import csv
 import dateutil.parser
 
 from django.db.models import Sum, Count, Min, Max, Q
-from django.http import HttpResponse, HttpResponseBadRequest, HttpResponseNotFound
+from django.http import HttpResponse, HttpResponseBadRequest, HttpResponseNotFound, HttpResponseForbidden
 from django.core.urlresolvers import reverse
 from django.contrib.auth.decorators import login_required, user_passes_test
 from locksmith.hub.models import Api, Key, Report, resolve_model
 from locksmith.hub.common import cycle_generator, exclude_internal_keys, exclude_internal_key_reports
 from unusual.http import BadRequest
 
-from django.conf import settings
-
 staff_required = user_passes_test(lambda u: u.is_staff)
 
 def _keys_issued_date_range():
@@ -118,6 +116,87 @@ def api_calls_daily(request):
     }
     return HttpResponse(content=json.dumps(result), status=200, content_type='application/json')
 
+@staff_required
+def calls_range(request):
+    if request.GET.has_key('begin'):
+        begin = request.GET['begin']
+
+        if request.GET.has_key('end'):
+            end = request.GET['end']
+            end_date = datetime.datetime.strptime(end, '%m/%d/%Y') 
+        else:
+            end_date = datetime.date()
+        #try:
+        begin_date = datetime.datetime.strptime(begin,'%m/%d/%Y')
+
+        resp = HttpResponse(status=200, content_type='text/csv')     
+        resp['Content-Disposition'] = 'attachment; filename="api_calls_%s_%s"' % (begin, end)
+
+        qry = Report.objects.filter(date__gte=begin_date, date__lte=end_date).extra(select={'day': 'date(date)'}).values('day').annotate(total=Count('date')).order_by('date')
+        writer = csv.writer(resp)
+        writer.writerow(['Day', 'Total Calls'])
+        for day in qry:
+            import logging; logging.debug(day)
+            writer.writerow([day['day'], day['total']])
+        return resp
+
+        #except:
+          #  return HttpResponse(content=json.dumps({"error": "Invalid Date Format"}), status=500, content_type='application/json')  
+
+
+@staff_required
+def all_calls(request):
+    ignore_deprecated = parse_bool_param(request, 'ignore_deprecated', False)
+    ignore_internal_keys = parse_bool_param(request, 'ignore_internal_keys', True)
+    if request.GET.has_key('year'):
+        year = int(request.GET['year'])
+        qry = Report.objects.filter(date__gte=datetime.date(year, 1, 1),
+                                    date__lte=datetime.date(year, 12, 31))
+        if ignore_deprecated == True:
+            qry = qry.filter(api__push_enabled=True)
+        if ignore_internal_keys:
+            qry = exclude_internal_key_reports(qry)
+        agg = qry.aggregate(calls=Sum('calls'))
+
+        daily_aggs = qry.values('date').annotate(calls=Sum('calls'))
+        monthly = dict(((m, {'month': m, 'calls': 0})
+                    for m in range(1, 13)))
+
+        for daily in daily_aggs:
+            month = daily['date'].month
+            monthly[month]['calls'] += daily['calls']
+
+        result = {
+            'calls': agg['calls'],
+            'monthly': monthly.values(),
+            'year': year
+        }
+        return HttpResponse(content=json.dumps(result), status=200, content_type='application/json')
+    else:
+        #yearly = Report.objects.raw("select id, extract(YEAR from  reported_time) as year, SUM(calls) as total from locksmith_hub_report group by extract(YEAR from reported_time)")
+
+        qry = Report.objects
+        if ignore_deprecated == True:
+            qry = qry.filter(api__push_enabled=True)
+        if ignore_internal_keys:
+            qry = exclude_internal_key_reports(qry)
+        daily_aggs = qry.values('date').annotate(calls=Sum('calls'))
+
+        yearly = {}
+        for daily in daily_aggs:
+            yr = daily['date'].year
+            if yr not in yearly:
+                yearly[yr] = {'year': yr, 'calls': 0}
+            yearly[yr]['calls'] += daily['calls']
+
+        result = {
+            'earliest_year': min(yearly.keys()),
+            'latest_year': max(yearly.keys()),
+            'yearly': yearly.values()
+        }
+        return HttpResponse(content=json.dumps(result), status=200, content_type='application/json')
+
+
 @login_required
 def calls_to_api(request,
                  api_id=None, api_name=None):
@@ -211,6 +290,8 @@ def calls_to_api_monthly(request,
 
     ignore_internal_keys = parse_bool_param(request, 'ignore_internal_keys', True)
     year = parse_int_param(request, 'year')
+    if year is None:
+        return HttpResponseBadRequest("You must specify a year parameter.")
 
     qry = Report.objects.filter(api=api)
     if ignore_internal_keys:
@@ -235,6 +316,45 @@ def calls_to_api_monthly(request,
     }
     return HttpResponse(content=json.dumps(result), status=200, content_type='application/json')
 
+
+@login_required
+def calls_to_api_daily(request,
+                         api_id=None, api_name=None):
+    if api_id is None and api_name is None:
+        return HttpResponseBadRequest('Must specify API id or name.')
+
+    try:
+        api = resolve_model(Api, [('id', api_id), ('name', api_name)])
+    except Api.DoesNotExist:
+        return HttpResponseNotFound('The requested API was not found.')
+
+    ignore_internal_keys = parse_bool_param(request, 'ignore_internal_keys', True)
+    end_date = parse_date_param(request, 'end_date')
+    begin_date = end_date - datetime.timedelta(days=7)
+
+    qry = Report.objects.filter(api=api)
+    if ignore_internal_keys:
+        qry = exclude_internal_key_reports(qry)
+    qry = qry.filter(date__gte=begin_date,
+                     date__lte=end_date)
+    agg = qry.aggregate(calls=Sum('calls'))
+
+    daily_aggs = qry.values('date').annotate(calls=Sum('calls'))
+
+    
+    result = {
+        'api_id': api.id,
+        'api_name': api.name,
+        'calls': agg['calls'],
+        'daily': []
+    }
+
+    for d in daily_aggs:
+        result['daily'].append({ 'date': d['date'].strftime('%m-%d-%Y'), 'calls': d['calls']})
+
+    return HttpResponse(content=json.dumps(result), status=200, content_type='application/json')
+
+
 @staff_required
 def keys(request):
     """Lists API keys. Compatible with jQuery DataTables."""
@@ -248,7 +368,11 @@ def keys(request):
     columns = ['key', 'email', 'calls', 'latest_call', 'issued_on']
     qry = Key.objects
     if sSearch not in (None, ''):
-        qry = qry.filter(Q(key__icontains=sSearch) | Q(email__icontains=sSearch))
+        qry = qry.filter(Q(key__icontains=sSearch)
+                         | Q(email__icontains=sSearch)
+                         | Q(name__icontains=sSearch)
+                         | Q(org_name__icontains=sSearch)
+                         | Q(org_url__icontains=sSearch))
     qry = qry.values('key', 'email', 'issued_on').annotate(calls=Sum('reports__calls'),
                                                            latest_call=Max('reports__date'))
     qry = qry.filter(calls__isnull=False)
@@ -335,6 +459,7 @@ def calls_by_endpoint(request, api_id=None, api_name=None):
     }
     return HttpResponse(content=json.dumps(result), status=200, content_type='application/json')
 
+@login_required
 def calls_from_key_yearly(request, key_uuid):
     try:
         key = Key.objects.get(key=key_uuid)
@@ -385,6 +510,9 @@ def calls_from_key_monthly(request, key_uuid):
         return HttpResponseForbidden()
 
     year = parse_int_param(request, 'year')
+    if year is None:
+        return HttpResponseBadRequest("You must specify a year parameter.")
+
     qry = key.reports.filter(date__gte=datetime.date(year, 1, 1),
                              date__lte=datetime.date(year, 12, 31))
     qry = qry.values('date').annotate(calls=Sum('calls'))
@@ -527,6 +655,8 @@ def keys_issued_yearly(request):
 @login_required
 def keys_issued_monthly(request):
     year = parse_int_param(request, 'year')
+    if year is None:
+        return HttpResponseBadRequest("You must specify a year parameter.")
     ignore_inactive = parse_bool_param(request, 'ignore_inactive', False)
     ignore_internal_keys = parse_bool_param(request, 'ignore_internal_keys', True)
 
