@@ -1,4 +1,4 @@
-from django.conf.urls.defaults import *
+from django.conf.urls import *
 
 urlpatterns = patterns('locksmith.mongoauth.views',
     url(r'^create_key/$', 'create_key', name='create_key'),
