@@ -1,4 +1,3 @@
-// Based on: http://bl.ocks.org/3766585
 function barChart () {
   var margin = {top: 50, right: 50, bottom: 50, left: 50},
       width = 420,
@@ -50,6 +49,7 @@ function barChart () {
           tooltip.style('visibility', 'hidden');
       };
 
+          
       // Select the svg element, if it exists.
       var svg = d3.select(this).selectAll("svg").data([data]);
 
@@ -83,8 +83,10 @@ function barChart () {
          .on('mouseover', show_tooltip)
          .on('mousemove', show_tooltip)
          .on('mouseout', hide_tooltip)
-         .on('click', hide_tooltip);
-
+         .on('click', hide_tooltip)
+        // .on('dblclick', toggle_stats);
+         
+      
     // x axis at the bottom of the chart
      g.select(".y.axis")
         .attr("transform", "translate(0, 0)")
@@ -229,7 +231,7 @@ function columnChart() {
       var hide_tooltip = function(d){
           tooltip.style('visibility', 'hidden');
       };
-
+      
       // Select the svg element, if it exists.
       var svg = d3.select(this).selectAll("svg").data([data]);
 
@@ -251,7 +253,8 @@ function columnChart() {
 
      // Update the bars.
       var bar = svg.select(".bars").selectAll(".bar").data(data);
-      bar.enter().append("rect");
+      bar.enter().append("rect").append("text").attr("x", function(d) {return X(d);}).attr("y", function(d, i){ return d[1] < 0 ? Y0() : Y(d); }).text(function(d){ return d;});
+
       bar.exit().remove();
       bar.attr("class", function(d, i) { return d[1] < 0 ? "bar negative" : "bar positive"; })
          .attr("x", function(d) { return X(d); })
@@ -263,7 +266,8 @@ function columnChart() {
          .on('mouseover', show_tooltip)
          .on('mousemove', show_tooltip)
          .on('mouseout', hide_tooltip)
-         .on('click', hide_tooltip);
+         .on('click', hide_tooltip)
+       //  .on('dblclick', toggle_stats);
 
     // x axis at the bottom of the chart
      g.select(".x.axis")
@@ -389,7 +393,7 @@ function ReactiveSettingsIface (target) {
     this.get = function (k) { return _setting(k); };
     this.set = function (k, v) { return _setting(k, v, true); };
     this.update = function (obj) {
-        console.log('Updating', $target.attr('id'), 'from', JSON.stringify(obj));
+        jQuery('#' + $target.attr('id')).find('.value').remove();
         for (var k in obj) {
             _setting(k, obj[k]);
         }
@@ -406,8 +410,40 @@ function ReactiveSettingsIface (target) {
         var obj = {};
         obj[k] = v;
         that.update(obj);
+
     });
 
+    $target.find('button.values').click(function(event){
+        jQuery(this).siblings('figure').find('rect.bar').each(function(index) { 
+          var d = jQuery(this);
+          if ($target.find('.bar_value_' + d.attr('data-independent')).length == 0){
+            var group = d.parents('figure')[0];
+            var offset = d.position();
+            var width = d.attr('width');
+           if (_settings['chart.type'] === 'bar'){
+              var top = parseInt(offset.top) + 5 + 'px'
+              var left = parseInt(offset.left) + parseInt(width) + 5 + 'px'
+           } else if (_settings['chart.type'] === 'column'){
+              var top = offset.top - 25 +  'px';
+              var left = offset.left + 10 + 'px';
+           }
+            if (d.attr('data-dependent') != 0) {
+              d3.select(group)
+                .append('div')
+                .attr('class', 'value bar_value_' + d.attr('data-independent'))
+                .style('visibility', 'visible')
+                .style('opacity', '1.0')
+                .style('position', 'absolute')
+                .style('top',  top)
+                .style('left', left)
+                .text(d.attr('data-dependent'));
+            }
+          } else {
+            $target.find('.bar_value_' + d.attr('data-independent')).remove();
+          }
+      })
+    })
+
     $target.find("button[data-setting]").each(function(){
         // Keep a lookup of the buttons for updating the UI later
         var k = $(this).attr("data-setting");
@@ -416,6 +452,7 @@ function ReactiveSettingsIface (target) {
         _buttons[k] = lst;
         $(this).click(function(event){
             that.buttons.trigger('click', this);
+
         });
     });
     this.buttons = $(_buttons);
@@ -441,7 +478,6 @@ function ReactiveSettingsHistoryIface (options) {
         if (options.compression_vocabulary) {
             decoded = vocab_translate_object(decoded, ANALYTICS_VOCAB);
         }
-        console.log("Decoded", anchor, " -> ", decoded);
         return decoded;
     };
 
@@ -476,7 +512,6 @@ function ReactiveSettingsHistoryIface (options) {
             encoded = vocab_translate_object(obj, ANALYTICS_VOCAB);
         }
         encoded = unicode_to_base64(JSON.stringify(encoded));
-        console.log("Encoded", obj, " -> ", encoded);
         return encoded;
     };
 
@@ -505,9 +540,7 @@ function ReactiveSettingsHistoryIface (options) {
     var _initial_popstate = true;
     var _initial_url = window.location.href;
     $(window).bind('popstate', function(event){
-        console.log('popstate');
         if ((_initial_popstate === true) && (window.location.href == _initial_popstate)) {
-            console.log("Ignoring initial popstate.");
             _initial_popstate = false;
             return;
         }
@@ -529,8 +562,6 @@ function ReactiveSettingsHistoryIface (options) {
         history.replaceState(encoded, null, window.location.href);
     }
 
-    console.log('ReactiveSettingsHistoryIface activated for',
-                keys(options.settings).join(', '));
     return that;
 };
 
@@ -585,7 +616,6 @@ function AnalyticsChart (options) {
     ReactiveSettingsIface.call(this, options['target']);
 
     var _data_callback = function(data){
-        console.log('Received data for', $target.attr('id'));
         _data = data;
         if (data.length === 0)
             _display_message('No data')
@@ -603,6 +633,7 @@ function AnalyticsChart (options) {
     };
 
     var _display_table = function(){
+
         var $table = $target.find("table.analytics-table");
         $table.find("tbody").empty();
         _data.forEach(function(pair){
@@ -703,15 +734,14 @@ function AnalyticsChart (options) {
 
     var _refresh = function(){
         _display_loading();
-        console.log('Refreshing for', $target.attr('id'));
 
         if (_data_deferred.state() == "pending") {
-            console.log('Rejecting deferred data');
             _data_deferred.reject();
         }
 
         _data_deferred = $.Deferred();
         _data_deferred.promise().then(_data_callback);
+
         opts['data_fn'].call(null, that, function(data){ _data_deferred.resolve(data); });
     };
     this.refresh = _refresh;
